@@ -1,30 +1,49 @@
 """Arbitrary precision arithmetic."""
 
-__version__ = '5.4.0'
+__version__ = '6.0.0'
 
 try:
     from ._rithm import (Endianness,
                          Fraction,
-                         Int)
+                         Int,
+                         TieBreaking)
 except ImportError:
     from enum import Enum as _Enum
     from math import gcd as _gcd
+    from numbers import (Integral as _Integral,
+                         Rational as _Rational)
     from operator import mul as _mul
-    from typing import (Any as _Any,
-                        Tuple as _Tuple,
-                        Union as _Union,
-                        overload as _overload)
+    from typing import (Tuple as _Tuple,
+                        Union as _Union)
 
 
-    class Endianness(_Enum):
-        BIG = 'big'
-        LITTLE = 'little'
-
+    class _BaseEnum(_Enum):
         def __repr__(self):
             return f'rithm.{type(self).__qualname__}.{self.name}'
 
 
+    class Endianness(_BaseEnum):
+        BIG = 'big'
+        LITTLE = 'little'
+
+
+    class TieBreaking(int, _BaseEnum):
+        AWAY_FROM_ZERO = 0
+        TO_EVEN = 1
+        TO_ODD = 2
+        TOWARD_ZERO = 3
+
+
+    @_Integral.register
     class Int:
+        @property
+        def denominator(self):
+            return _ONE
+
+        @property
+        def numerator(self):
+            return self
+
         def bit_length(self):
             return Int(self._value.bit_length())
 
@@ -56,12 +75,12 @@ except ImportError:
         def __add__(self, other):
             return (Int(self._value + other._value)
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else self.__radd__(other))
 
         def __and__(self, other):
             return (Int(self._value & other._value)
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else self.__rand__(other))
 
         def __bool__(self):
             return bool(self._value)
@@ -73,12 +92,17 @@ except ImportError:
             return (tuple(Int(value)
                           for value in divmod(self._value, other._value))
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else (tuple(Int(value)
+                                for value in divmod(self._value, other))
+                          if isinstance(other, int)
+                          else NotImplemented))
 
         def __eq__(self, other):
             return (self._value == other._value
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else (self._value == other
+                          if isinstance(other, int)
+                          else NotImplemented))
 
         def __float__(self):
             return float(self._value)
@@ -89,12 +113,16 @@ except ImportError:
         def __floordiv__(self, other):
             return (Int(self._value // other._value)
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else (Int(self._value // other)
+                          if isinstance(other, int)
+                          else NotImplemented))
 
         def __ge__(self, other):
             return (self._value >= other._value
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else (self._value >= other
+                          if isinstance(other, int)
+                          else NotImplemented))
 
         def __getstate__(self):
             return self._value
@@ -102,7 +130,9 @@ except ImportError:
         def __gt__(self, other):
             return (self._value > other._value
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else (self._value > other
+                          if isinstance(other, int)
+                          else NotImplemented))
 
         def __hash__(self):
             return hash(self._value)
@@ -116,27 +146,35 @@ except ImportError:
         def __le__(self, other):
             return (self._value <= other._value
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else (self._value <= other
+                          if isinstance(other, int)
+                          else NotImplemented))
 
         def __lshift__(self, other):
             return (Int(self._value << other._value)
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else (Int(self._value << other)
+                          if isinstance(other, int)
+                          else NotImplemented))
 
         def __lt__(self, other):
             return (self._value < other._value
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else (self._value < other
+                          if isinstance(other, int)
+                          else NotImplemented))
 
         def __mod__(self, other):
             return (Int(self._value % other._value)
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else (Int(self._value % other)
+                          if isinstance(other, int)
+                          else NotImplemented))
 
         def __mul__(self, other):
             return (Int(self._value * other._value)
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else self.__rmul__(other))
 
         def __neg__(self):
             return Int(-self._value)
@@ -144,26 +182,98 @@ except ImportError:
         def __or__(self, other):
             return (Int(self._value | other._value)
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else self.__ror__(other))
+
+        def __pos__(self):
+            return self
 
         def __pow__(self, exponent, divisor=None):
-            return (((Int(self._value ** exponent._value)
+            return (((Int(self._value ** int(exponent))
                       if exponent >= _ZERO
                       else Fraction(_ONE, self) ** -exponent)
                      if divisor is None
-                     else (Int(pow(self._value, exponent._value,
-                                   divisor._value))
-                           if isinstance(divisor, Int)
+                     else (Int(pow(self._value, int(exponent), int(divisor)))
+                           if isinstance(divisor, (Int, int))
                            else NotImplemented))
-                    if isinstance(exponent, Int)
+                    if isinstance(exponent, (Int, int))
+                    else NotImplemented)
+
+        def __radd__(self, other):
+            return (Int(self._value + other)
+                    if isinstance(other, int)
+                    else NotImplemented)
+
+        def __rand__(self, other):
+            return (Int(self._value & other)
+                    if isinstance(other, int)
+                    else NotImplemented)
+
+        def __rdivmod__(self, other):
+            return (tuple(Int(value) for value in divmod(other, self._value))
+                    if isinstance(other, int)
                     else NotImplemented)
 
         def __repr__(self):
-            return f'rithm.Int(\'{self._value}\')'
+            return f'rithm.Int({self._value})'
+
+        def __rfloordiv__(self, other):
+            return (Int(other // self._value)
+                    if isinstance(other, int)
+                    else NotImplemented)
+
+        def __rlshift__(self, other):
+            return (Int(other << self._value)
+                    if isinstance(other, int)
+                    else NotImplemented)
+
+        def __rmod__(self, other):
+            return (Int(other % self._value)
+                    if isinstance(other, int)
+                    else NotImplemented)
+
+        def __rmul__(self, other):
+            return (Int(self._value * other)
+                    if isinstance(other, int)
+                    else NotImplemented)
+
+        def __ror__(self, other):
+            return (Int(self._value | other)
+                    if isinstance(other, int)
+                    else NotImplemented)
+
+        def __round__(self, digits=None):
+            return Int(round(self._value, digits))
+
+        def __rpow__(self, base, divisor=None):
+            return (Int(base).__pow__(self)
+                    if isinstance(base, int)
+                    else NotImplemented)
+
+        def __rrshift__(self, other):
+            return (Int(other >> self._value)
+                    if isinstance(other, int)
+                    else NotImplemented)
 
         def __rshift__(self, other):
             return (Int(self._value >> other._value)
                     if isinstance(other, Int)
+                    else (Int(self._value >> other)
+                          if isinstance(other, int)
+                          else NotImplemented))
+
+        def __rsub__(self, other):
+            return (Int(other - self._value)
+                    if isinstance(other, int)
+                    else NotImplemented)
+
+        def __rtruediv__(self, other):
+            return (Fraction(other, self)
+                    if isinstance(other, int)
+                    else NotImplemented)
+
+        def __rxor__(self, other):
+            return (Int(self._value ^ other)
+                    if isinstance(other, int)
                     else NotImplemented)
 
         def __setstate__(self, state: int):
@@ -175,11 +285,13 @@ except ImportError:
         def __sub__(self, other):
             return (Int(self._value - other._value)
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else (Int(self._value - other)
+                          if isinstance(other, int)
+                          else NotImplemented))
 
         def __truediv__(self, other):
             return (Fraction(self, other)
-                    if isinstance(other, Int)
+                    if isinstance(other, (Int, int))
                     else NotImplemented)
 
         def __trunc__(self):
@@ -188,7 +300,7 @@ except ImportError:
         def __xor__(self, other):
             return (Int(self._value ^ other._value)
                     if isinstance(other, Int)
-                    else NotImplemented)
+                    else self.__rxor__(other))
 
 
     def _to_bytes_count(value: int) -> int:
@@ -199,6 +311,7 @@ except ImportError:
     _ZERO = Int()
 
 
+    @_Rational.register
     class Fraction:
         @property
         def denominator(self):
@@ -208,6 +321,26 @@ except ImportError:
         def numerator(self):
             return self._numerator
 
+        def round(self, tie_breaking):
+            quotient, remainder = divmod(self.numerator, self.denominator)
+            double_remainder = remainder * 2
+            if double_remainder == self.denominator:
+                if tie_breaking is TieBreaking.AWAY_FROM_ZERO:
+                    return quotient + _ONE if quotient >= 0 else quotient
+                elif tie_breaking is TieBreaking.TO_EVEN:
+                    return quotient + _ONE if quotient % 2 else quotient
+                elif tie_breaking is TieBreaking.TO_ODD:
+                    return quotient + _ONE if not quotient % 2 else quotient
+                else:
+                    assert tie_breaking is TieBreaking.TOWARD_ZERO, (
+                        tie_breaking
+                    )
+                    return quotient + _ONE if quotient < 0 else quotient
+            else:
+                return (quotient + _ONE
+                        if double_remainder > self.denominator
+                        else quotient)
+
         __slots__ = '_denominator', '_numerator'
 
         def __new__(cls,
@@ -217,7 +350,10 @@ except ImportError:
                     _normalize=True):
             self = super().__new__(cls)
             if _denominator is None:
-                if isinstance(_numerator, float):
+                if isinstance(_numerator, Fraction):
+                    numerator, denominator = (_numerator.numerator,
+                                              _numerator.denominator)
+                elif isinstance(_numerator, float):
                     raw_numerator, raw_denominator = (
                         _numerator.as_integer_ratio()
                     )
@@ -225,6 +361,9 @@ except ImportError:
                                               Int(raw_denominator))
                 elif isinstance(_numerator, (Int, int)):
                     numerator, denominator = Int(_numerator), _ONE
+                elif isinstance(_numerator, _Rational):
+                    numerator, denominator = (_numerator.numerator,
+                                              _numerator.denominator)
                 else:
                     raise TypeError('First argument should be of '
                                     f'type {Int}, {int} or {float}, '
@@ -252,129 +391,104 @@ except ImportError:
             return Fraction(abs(self.numerator), self.denominator,
                             _normalize=False)
 
-        @_overload
-        def __add__(self, other: _Any) -> _Any:
-            ...
-
-        @_overload
-        def __add__(self, other: _Union['Fraction', Int]):
-            ...
-
-        def __add__(self, other: _Union['Fraction', Int, _Any]
-                    ) -> _Union['Fraction', _Any]:
+        def __add__(self, other):
             return (self._add_fraction(other)
                     if isinstance(other, Fraction)
-                    else (self._add_int(other)
-                          if isinstance(other, Int)
-                          else NotImplemented))
+                    else self.__radd__(other))
 
         def __bool__(self):
             return bool(self.numerator)
 
-        def __eq__(self, other: 'Fraction'):
+        def __ceil__(self):
+            return -(-self.numerator // self.denominator)
+
+        def __divmod__(self, divisor):
+            return (_divmod_rationals(self, divisor)
+                    if isinstance(divisor, (Fraction, Int, int))
+                    else NotImplemented)
+
+        def __eq__(self, other):
             return (self.numerator == other.numerator
                     and self.denominator == other.denominator
                     if isinstance(other, Fraction)
                     else (self.denominator == _ONE
                           and self.numerator == other
-                          if isinstance(other, Int)
+                          if isinstance(other, (Int, int))
                           else NotImplemented))
 
         def __float__(self):
-            return self._numerator._value / self._denominator._value
+            return int(self.numerator) / int(self.denominator)
 
-        @_overload
-        def __floordiv__(self, other: _Any) -> _Any:
-            ...
+        def __floor__(self):
+            return self.numerator // self.denominator
 
-        @_overload
-        def __floordiv__(self, other: _Union['Fraction', Int]):
-            ...
-
-        def __floordiv__(self, other: _Union['Fraction', Int, _Any]
-                         ) -> _Union[Int, _Any]:
-            return ((self.numerator * other.denominator)
-                    // (self.denominator * other.numerator)
-                    if isinstance(other, Fraction)
-                    else (self.numerator // (self.denominator * other)
-                          if isinstance(other, Int)
+        def __floordiv__(self, divisor):
+            return ((self.numerator * divisor.denominator)
+                    // (self.denominator * divisor.numerator)
+                    if isinstance(divisor, Fraction)
+                    else (self.numerator // (self.denominator * divisor)
+                          if isinstance(divisor, (Int, int))
                           else NotImplemented))
 
-        def __ge__(self, other: 'Fraction'):
+        def __ge__(self, other):
             return (self.numerator * other.denominator
                     >= other.numerator * self.denominator
                     if isinstance(other, Fraction)
                     else (self.numerator >= other * self.denominator
-                          if isinstance(other, Int)
+                          if isinstance(other, (Int, int))
                           else NotImplemented))
 
-        def __getstate__(self) -> _Tuple[Int, Int]:
+        def __getstate__(self):
             return self._numerator, self._denominator
 
-        def __gt__(self, other: 'Fraction'):
+        def __gt__(self, other):
             return (self.numerator * other.denominator
                     > other.numerator * self.denominator
                     if isinstance(other, Fraction)
                     else (self.numerator > other * self.denominator
-                          if isinstance(other, Int)
+                          if isinstance(other, (Int, int))
                           else NotImplemented))
 
-        def __le__(self, other: 'Fraction'):
+        def __le__(self, other):
             return (self.numerator * other.denominator
                     <= other.numerator * self.denominator
                     if isinstance(other, Fraction)
                     else (self.numerator <= other * self.denominator
-                          if isinstance(other, Int)
+                          if isinstance(other, (Int, int))
                           else NotImplemented))
 
-        def __lt__(self, other: 'Fraction'):
+        def __lt__(self, other):
             return (self.numerator * other.denominator
                     < other.numerator * self.denominator
                     if isinstance(other, Fraction)
                     else (self.numerator < other * self.denominator
-                          if isinstance(other, Int)
+                          if isinstance(other, (Int, int))
                           else NotImplemented))
 
-        @_overload
-        def __mod__(self, other: _Any) -> _Any:
-            ...
-
-        @_overload
-        def __mod__(self, other: _Union['Fraction', Int]):
-            ...
-
-        def __mod__(self, other: _Union['Fraction', Int, _Any]
-                    ) -> _Union['Fraction', _Any]:
-            return (Fraction((self.numerator * other.denominator)
-                             % (self.denominator * other.numerator),
-                             self.denominator * other.denominator)
-                    if isinstance(other, Fraction)
-                    else (Fraction(self.numerator % (self.denominator * other),
+        def __mod__(self, divisor):
+            return (Fraction((self.numerator * divisor.denominator)
+                             % (self.denominator * divisor.numerator),
+                             self.denominator * divisor.denominator)
+                    if isinstance(divisor, Fraction)
+                    else (Fraction(self.numerator
+                                   % (self.denominator * divisor),
                                    self.denominator)
-                          if isinstance(other, Int)
+                          if isinstance(divisor, (Int, int))
                           else NotImplemented))
 
-        @_overload
-        def __mul__(self, other: _Any) -> _Any:
-            ...
-
-        @_overload
-        def __mul__(self, other: _Union['Fraction', Int]):
-            ...
-
-        def __mul__(self, other: _Union['Fraction', Int, _Any]
-                    ) -> _Union['Fraction', _Any]:
+        def __mul__(self, other):
             return (self._mul_by_fraction(other)
                     if isinstance(other, Fraction)
-                    else (self._mul_by_int(other)
-                          if isinstance(other, Int)
-                          else NotImplemented))
+                    else self.__rmul__(other))
 
         def __neg__(self):
             return Fraction(-self.numerator, self.denominator,
                             _normalize=False)
 
-        def __pow__(self, exponent, divisor: None = None):
+        def __pos__(self):
+            return self
+
+        def __pow__(self, exponent, divisor=None):
             return ((Fraction(self.numerator ** exponent,
                               self.denominator ** exponent,
                               _normalize=False)
@@ -387,90 +501,63 @@ except ImportError:
                              ),
                              _normalize=False
                      ))
-                    if isinstance(exponent, Int) and divisor is None
+                    if isinstance(exponent, (Int, int)) and divisor is None
+                    else NotImplemented)
+
+        def __radd__(self, other):
+            return (self._add_int(Int(other))
+                    if isinstance(other, (Int, int))
+                    else NotImplemented)
+
+        def __rdivmod__(self, dividend):
+            return (_divmod_rationals(dividend, self)
+                    if isinstance(dividend, (Int, int))
                     else NotImplemented)
 
         def __repr__(self):
             return f'rithm.Fraction({self.numerator!r}, {self.denominator!r})'
 
-        @_overload
-        def __radd__(self, other: _Any) -> _Any:
-            ...
-
-        @_overload
-        def __radd__(self, other: Int):
-            ...
-
-        def __radd__(self, other: _Union[Int, _Any]
-                     ) -> _Union['Fraction', _Any]:
-            return (self._add_int(other)
-                    if isinstance(other, Int)
+        def __rfloordiv__(self, dividend):
+            return ((dividend * self.denominator) // self.numerator
+                    if isinstance(dividend, (Int, int))
                     else NotImplemented)
 
-        @_overload
-        def __rfloordiv__(self, other: _Any) -> _Any:
-            ...
-
-        @_overload
-        def __rfloordiv__(self, other: Int):
-            ...
-
-        def __rfloordiv__(self, other: _Union[Int, _Any]) -> _Union[Int, _Any]:
-            return ((other * self.denominator) // self.numerator
-                    if isinstance(other, Int)
-                    else NotImplemented)
-
-        @_overload
-        def __rmod__(self, other: _Any) -> _Any:
-            ...
-
-        @_overload
-        def __rmod__(self, other: Int):
-            ...
-
-        def __rmod__(self, other: _Union[Int, _Any]
-                     ) -> _Union['Fraction', _Any]:
-            return (Fraction((other * self.denominator) % self.numerator,
+        def __rmod__(self, dividend):
+            return (Fraction((dividend * self.denominator) % self.numerator,
                              self.denominator)
-                    if isinstance(other, Int)
+                    if isinstance(dividend, (Int, int))
                     else NotImplemented)
 
-        @_overload
-        def __rmul__(self, other: _Any) -> _Any:
-            ...
-
-        @_overload
-        def __rmul__(self, other: Int):
-            ...
-
-        def __rmul__(self, other: _Union[Int, _Any]
-                     ) -> _Union['Fraction', _Any]:
-            return (self._mul_by_int(other)
-                    if isinstance(other, Int)
+        def __rmul__(self, other):
+            return (self._mul_by_int(Int(other))
+                    if isinstance(other, (Int, int))
                     else NotImplemented)
 
-        @_overload
-        def __rsub__(self, other: _Any) -> _Any:
-            ...
+        def __round__(self, digits=None):
+            if digits is None:
+                return self.round(TieBreaking.TO_EVEN)
+            else:
+                shift = 10 ** abs(digits)
+                return (Fraction((self * shift).round(TieBreaking.TO_EVEN),
+                                 shift)
+                        if digits > 0
+                        else Fraction((self / shift).round(TieBreaking.TO_EVEN)
+                                      * shift))
 
-        @_overload
-        def __rsub__(self, other: Int):
-            ...
-
-        def __rsub__(self, other: _Union[Int, _Any]
-                     ) -> _Union['Fraction', _Any]:
+        def __rsub__(self, subtrahend):
             return (
                 Fraction(
-                        *_normalize_components_moduli(other * self.denominator
-                                                      - self.numerator,
-                                                      self.denominator),
+                        *_normalize_components_moduli(
+                                subtrahend * self.denominator - self.numerator,
+                                self.denominator
+                        ),
                         _normalize=False
                 )
-                if isinstance(other, Int)
+                if isinstance(subtrahend, (Int, int))
                 else NotImplemented
             )
 
-        def __setstate__(self, state: _Tuple[Int, Int]):
+        def __setstate__(self, state):
             self._numerator, self._denominator = state
 
         def __str__(self):
@@ -478,80 +565,58 @@ except ImportError:
                     if self.denominator == _ONE
                     else f'{self.numerator}/{self.denominator}')
 
-        @_overload
-        def __sub__(self, other: _Any) -> _Any:
-            ...
-
-        @_overload
-        def __sub__(self, other: _Union['Fraction', Int]):
-            ...
-
-        def __sub__(self, other: _Union['Fraction', Int, _Any]
-                    ) -> _Union['Fraction', _Any]:
+        def __sub__(self, minuend):
             return (Fraction(
                     *_normalize_components_moduli(
-                            self.numerator * other.denominator
-                            - other.numerator * self.denominator,
-                            self.denominator * other.denominator
+                            self.numerator * minuend.denominator
+                            - minuend.numerator * self.denominator,
+                            self.denominator * minuend.denominator
                     ),
                     _normalize=False)
-                    if isinstance(other, Fraction)
+                    if isinstance(minuend, Fraction)
                     else
                     (Fraction(
                             *_normalize_components_moduli(
-                                    self.numerator - other * self.denominator,
+                                    self.numerator - minuend * self.denominator,
                                     self.denominator
                             ),
                             _normalize=False
                     )
-                     if isinstance(other, Int)
+                     if isinstance(minuend, (Int, int))
                      else NotImplemented))
 
-        @_overload
-        def __rtruediv__(self, other: _Any) -> _Any:
-            ...
-
-        @_overload
-        def __rtruediv__(self, other: Int):
-            ...
-
-        def __rtruediv__(self, other: _Union[Int, _Any]
-                         ) -> _Union['Fraction', _Any]:
-            return (self._rtruediv_by_int(other)
-                    if isinstance(other, Int)
+        def __rtruediv__(self, dividend):
+            return (self._rtruediv_by_int(Int(dividend))
+                    if isinstance(dividend, (Int, int))
                     else NotImplemented)
 
-        @_overload
-        def __truediv__(self, other: _Any) -> _Any:
-            ...
-
-        @_overload
-        def __truediv__(self, other: _Union['Fraction', Int]):
-            ...
-
-        def __truediv__(self, other: _Union['Fraction', Int, _Any]
-                        ) -> _Union['Fraction', _Any]:
+        def __truediv__(self, divisor):
             return (
                 Fraction(
                         *_normalize_components_sign(
                                 *map(_mul,
                                      _normalize_components_moduli(
-                                             self.numerator, other.numerator
+                                             self.numerator, divisor.numerator
                                      ),
                                      _normalize_components_moduli(
-                                             other.denominator,
+                                             divisor.denominator,
                                              self.denominator
                                      ))
                         ),
                         _normalize=False
                 )
-                if isinstance(other, Fraction)
-                else (self._truediv_by_int(other)
-                      if isinstance(other, Int)
+                if isinstance(divisor, Fraction)
+                else (self._truediv_by_int(Int(divisor))
+                      if isinstance(divisor, (Int, int))
                       else NotImplemented)
             )
 
-        def _add_fraction(self, other: 'Fraction'):
+        def __trunc__(self):
+            return (self.__ceil__()
+                    if self.numerator < _ZERO
+                    else self.__floor__())
+
+        def _add_fraction(self, other: 'Fraction') -> 'Fraction':
             return Fraction(
                     *_normalize_components_moduli(
                             self.numerator * other.denominator
@@ -561,7 +626,7 @@ except ImportError:
                     _normalize=False
             )
 
-        def _add_int(self, other: Int):
+        def _add_int(self, other: Int) -> 'Fraction':
             return Fraction(
                     *_normalize_components_moduli(self.numerator
                                                   + other * self.denominator,
@@ -569,7 +634,7 @@ except ImportError:
                     _normalize=False
             )
 
-        def _mul_by_fraction(self, other: 'Fraction'):
+        def _mul_by_fraction(self, other: 'Fraction') -> 'Fraction':
             numerator, other_denominator = _normalize_components_moduli(
                     self.numerator, other.denominator)
             other_numerator, denominator = _normalize_components_moduli(
@@ -578,39 +643,50 @@ except ImportError:
                             denominator * other_denominator,
                             _normalize=False)
 
-        def _mul_by_int(self, other: Int):
+        def _mul_by_int(self, other: Int) -> 'Fraction':
             other, denominator = _normalize_components_moduli(other,
                                                               self.denominator)
             return Fraction(self.numerator * other, denominator,
                             _normalize=False)
 
-        def _rtruediv_by_int(self, other: Int):
-            other, numerator = _normalize_components_moduli(other,
-                                                            self.numerator)
+        def _rtruediv_by_int(self, dividend: Int) -> 'Fraction':
+            dividend, numerator = _normalize_components_moduli(dividend,
+                                                               self.numerator)
             return Fraction(
-                    *_normalize_components_sign(other * self.denominator,
+                    *_normalize_components_sign(dividend * self.denominator,
                                                 numerator),
                     _normalize=False
             )
 
-        def _truediv_by_int(self, other: Int):
-            numerator, other = _normalize_components_moduli(self.numerator,
-                                                            other)
+        def _truediv_by_int(self, divisor: Int) -> 'Fraction':
+            numerator, divisor = _normalize_components_moduli(self.numerator,
+                                                              divisor)
             return Fraction(
                     *_normalize_components_sign(numerator,
-                                                other * self.denominator),
+                                                divisor * self.denominator),
                     _normalize=False
             )
 
 
-    def _normalize_components_moduli(numerator: Int,
-                                     denominator: Int) -> _Tuple[Int, Int]:
+    def _divmod_rationals(dividend: _Union[Fraction, Int, int],
+                          divisor: _Union[Fraction, Int, int]
+                          ) -> _Tuple[Int, Fraction]:
+        quotient, remainder_numerator = divmod(
+                dividend.numerator * divisor.denominator,
+                dividend.denominator * divisor.numerator
+        )
+        return quotient, Fraction(remainder_numerator,
+                                  dividend.denominator * divisor.denominator)
+
+
+    def _normalize_components_moduli(numerator: Int, denominator: Int
+                                     ) -> _Tuple[Int, Int]:
         gcd = numerator.gcd(denominator)
         return numerator // gcd, denominator // gcd
 
 
-    def _normalize_components_sign(numerator: Int,
-                                   denominator: Int) -> _Tuple[Int, Int]:
+    def _normalize_components_sign(numerator: Int, denominator: Int
+                                   ) -> _Tuple[Int, Int]:
         return ((-numerator, -denominator)
                 if denominator < _ZERO
                 else (numerator, denominator))
