@@ -15,7 +15,7 @@ use crate::traits::{
     CheckedDivAsF32, CheckedDivAsF64, CheckedDivEuclid, CheckedDivRem, CheckedDivRemEuclid,
     CheckedPow, CheckedPowRemEuclid, CheckedRem, CheckedRemEuclid, CheckedRemEuclidInv, CheckedShl,
     CheckedShr, DivEuclid, DivRem, DivRemEuclid, DoublePrecisionOf, Endianness, Float, FromBytes,
-    FromStrRadix, Gcd, Oppose, OppositionOf, Oppositive, Pow, RemEuclid, ToBytes, Unitary,
+    FromStrRadix, Gcd, Oppose, OppositionOf, Oppositive, Parity, Pow, RemEuclid, ToBytes, Unitary,
     Zeroable,
 };
 use crate::utils;
@@ -26,8 +26,10 @@ pub struct BigInt<Digit, const SEPARATOR: char, const SHIFT: usize> {
     digits: Vec<Digit>,
 }
 
-const MAX_REPRESENTABLE_BASE: u8 = 36;
+pub(crate) const MIN_REPRESENTABLE_BASE: u8 = 2;
+pub(crate) const MAX_REPRESENTABLE_BASE: u8 = 36;
 
+#[derive(Eq, PartialEq)]
 pub enum FromFloatConversionError {
     Infinity,
     NaN,
@@ -54,21 +56,24 @@ impl Display for FromFloatConversionError {
     }
 }
 
+#[derive(Eq, PartialEq)]
 pub enum FromStringConversionError {
-    StartsWithSeparator,
+    BaseOutOfBounds(u32),
     ConsecutiveSeparators,
-    InvalidDigit(char, u8),
     EndsWithSeparator,
+    InvalidDigit(char, u8),
+    StartsWithSeparator,
 }
 
 impl FromStringConversionError {
     fn description(&self) -> String {
         match self {
-            FromStringConversionError::StartsWithSeparator => {
-                String::from("Should not start with separator.")
-            }
-            FromStringConversionError::InvalidDigit(character, base) => {
-                format!("Invalid digit in base {}: {}.", base, character)
+            FromStringConversionError::BaseOutOfBounds(base) => {
+                format!(
+                    "Base should be zero or in range from {MIN_REPRESENTABLE_BASE} \
+                     to {MAX_REPRESENTABLE_BASE}, but found: {}.",
+                    base
+                )
             }
             FromStringConversionError::ConsecutiveSeparators => {
                 String::from("Consecutive separators found.")
@@ -76,6 +81,12 @@ impl FromStringConversionError {
             FromStringConversionError::EndsWithSeparator => {
                 String::from("Should not end with separator.")
             }
+            FromStringConversionError::InvalidDigit(character, base) => {
+                format!("Invalid digit in base {}: {:?}.", base, character)
+            }
+            FromStringConversionError::StartsWithSeparator => {
+                String::from("Should not start with separator.")
+            }
         }
     }
 }
@@ -92,6 +103,7 @@ impl Display for FromStringConversionError {
     }
 }
 
+#[derive(Eq, PartialEq)]
 pub enum LeftShiftError {
     NegativeShift,
     OutOfMemory,
@@ -120,6 +132,7 @@ impl Display for LeftShiftError {
     }
 }
 
+#[derive(Eq, PartialEq)]
 pub enum RightShiftError {
     NegativeShift,
 }
@@ -144,6 +157,7 @@ impl Display for RightShiftError {
     }
 }
 
+#[derive(Eq, PartialEq)]
 pub enum ToFloatConversionError {
     TooLarge,
 }
@@ -189,6 +203,9 @@ impl<Digit: FromStrDigit, const SEPARATOR: char, const SHIFT: usize>
     fn new(string: &str, mut base: u8) -> Result<Self, FromStringConversionError> {
         debug_assert!(is_valid_shift::<Digit, SHIFT>());
         debug_assert!(Self::ASCII_CODES_DIGIT_VALUES[SEPARATOR as usize] >= MAX_REPRESENTABLE_BASE);
+        debug_assert!(
+            base == 0 || (base >= MIN_REPRESENTABLE_BASE && base <= MAX_REPRESENTABLE_BASE)
+        );
         let mut characters = string.trim().chars().peekable();
         let sign = Self::parse_sign(&mut characters);
         if base == 0 {
@@ -378,8 +395,8 @@ where
             negate_digits(&mut result);
         }
         match endianness {
-            Endianness::BIG => result.reverse(),
-            Endianness::LITTLE => {}
+            Endianness::Big => result.reverse(),
+            Endianness::Little => {}
         }
         result
     }
@@ -393,8 +410,8 @@ where
     fn from_bytes(bytes: &[u8], endianness: Endianness) -> Self {
         let mut bytes = bytes.to_vec();
         match endianness {
-            Endianness::BIG => bytes.reverse(),
-            Endianness::LITTLE => {}
+            Endianness::Big => bytes.reverse(),
+            Endianness::Little => {}
         }
         debug_assert!(is_valid_shift::<Digit, SHIFT>());
         let most_significant_byte = bytes[bytes.len() - 1];
@@ -419,8 +436,8 @@ where
 impl<Digit, const SEPARATOR: char, const SHIFT: usize> Abs for BigInt<Digit, SEPARATOR, SHIFT> {
     type Output = Self;
 
-    fn abs(self) -> Self {
-        Self {
+    fn abs(self) -> Self::Output {
+        Self::Output {
             sign: self.sign.abs(),
             digits: self.digits,
         }
@@ -435,7 +452,7 @@ impl<Digit: AdditiveDigit, const SEPARATOR: char, const SHIFT: usize> Add
     fn add(self, other: Self) -> Self::Output {
         let (sign, digits) =
             sum_signed_digits::<Digit, SHIFT>(&self.digits, self.sign, &other.digits, other.sign);
-        Self { sign, digits }
+        Self::Output { sign, digits }
     }
 }
 
@@ -459,7 +476,7 @@ impl<Digit: BinaryDigit, const SEPARATOR: char, const SHIFT: usize> BitAnd
         } else {
             bitwise_and::<Digit, SHIFT>(other.digits, other.sign, self.digits, self.sign)
         };
-        Self { sign, digits }
+        Self::Output { sign, digits }
     }
 }
 
@@ -506,7 +523,7 @@ impl<Digit: BinaryDigit, const SEPARATOR: char, const SHIFT: usize> BitOr
         } else {
             bitwise_or::<Digit, SHIFT>(other.digits, other.sign, self.digits, self.sign)
         };
-        Self { sign, digits }
+        Self::Output { sign, digits }
     }
 }
 
@@ -533,7 +550,7 @@ impl<Digit: BinaryDigit, const SEPARATOR: char, const SHIFT: usize> BitXor
         } else {
             bitwise_xor::<Digit, SHIFT>(other.digits, other.sign, self.digits, self.sign)
         };
-        Self { sign, digits }
+        Self::Output { sign, digits }
     }
 }
 
@@ -647,66 +664,10 @@ impl<Digit: ExponentiativeDigit, const SEPARATOR: char, const SHIFT: usize> Chec
 
     fn checked_pow(self, exponent: Self) -> Self::Output {
         if exponent.is_negative() {
-            return None;
-        }
-        let mut result = Self::one();
-        let mut exponent_digit = exponent.digits[exponent.digits.len() - 1];
-        if exponent.digits.len() == 1 && exponent_digit <= Digit::from(3) {
-            if exponent_digit >= Digit::from(2) {
-                result = self.clone() * self.clone();
-                if exponent_digit == Digit::from(3) {
-                    result *= self;
-                }
-            } else if exponent_digit.is_one() {
-                result *= self;
-            }
-        } else if exponent.digits.len() <= WINDOW_CUTOFF {
-            result = self.clone();
-            let mut exponent_digit_mask = Digit::from(2);
-            loop {
-                if exponent_digit_mask > exponent_digit {
-                    exponent_digit_mask >>= 1;
-                    break;
-                }
-                exponent_digit_mask <<= 1;
-            }
-            exponent_digit_mask >>= 1;
-            let mut exponent_digits_iterator = exponent.digits.iter().rev().skip(1).peekable();
-            loop {
-                while !exponent_digit_mask.is_zero() {
-                    result *= result.clone();
-                    if !(exponent_digit & exponent_digit_mask).is_zero() {
-                        result *= self.clone();
-                    }
-                    exponent_digit_mask >>= 1;
-                }
-                if exponent_digits_iterator.peek().is_none() {
-                    break;
-                }
-                exponent_digit = unsafe { *exponent_digits_iterator.next().unwrap_unchecked() };
-                exponent_digit_mask = Digit::one() << (SHIFT - 1);
-            }
+            None
         } else {
-            let mut cache = vec![Self::zero(); WINDOW_BASE];
-            cache[0] = result.clone();
-            for index in 1..WINDOW_BASE {
-                cache[index] = cache[index - 1].clone() * self.clone();
-            }
-            let exponent_window_digits = binary_digits_to_lesser_binary_base::<Digit, WindowDigit>(
-                &exponent.digits,
-                SHIFT,
-                WINDOW_SHIFT,
-            );
-            for &digit in exponent_window_digits.iter().rev() {
-                for _ in 0..WINDOW_SHIFT {
-                    result *= result.clone();
-                }
-                if !digit.is_zero() {
-                    result *= cache[digit as usize].clone();
-                }
-            }
+            Some(self.unchecked_pow(&exponent))
         }
-        Some(result)
     }
 }
 
@@ -869,7 +830,7 @@ impl<Digit: DivisibleDigit, const SEPARATOR: char, const SHIFT: usize> Div
             divisor.sign,
         )
         .unwrap();
-        Self { sign, digits }
+        Self::Output { sign, digits }
     }
 }
 
@@ -928,7 +889,7 @@ impl<Digit: EuclidDivisibleDigit, const SEPARATOR: char, const SHIFT: usize> Div
             divisor.sign,
         )
         .unwrap();
-        Self { sign, digits }
+        Self::Output { sign, digits }
     }
 }
 
@@ -1039,13 +1000,13 @@ impl<Digit: FromStrDigit, const SEPARATOR: char, const SHIFT: usize> FromStrRadi
     type Error = FromStringConversionError;
 
     fn from_str_radix(string: &str, radix: u32) -> Result<Self, Self::Error> {
-        if (radix != 0 && radix < 2) || radix > (MAX_REPRESENTABLE_BASE as u32) {
-            panic!(
-                "Radix should be in range from 2 to {}.",
-                MAX_REPRESENTABLE_BASE
-            );
+        if radix != 0
+            && (radix < (MIN_REPRESENTABLE_BASE as u32) || radix > (MAX_REPRESENTABLE_BASE as u32))
+        {
+            Err(FromStringConversionError::BaseOutOfBounds(radix))
+        } else {
+            Self::new(string, radix as u8)
         }
-        Self::new(string, radix as u8)
     }
 }
 
@@ -1054,7 +1015,7 @@ impl<Digit: GcdDigit, const SEPARATOR: char, const SHIFT: usize> Gcd
 {
     type Output = Self;
 
-    fn gcd(self, other: Self) -> Self {
+    fn gcd(self, other: Self) -> Self::Output {
         let mut largest_digits = self.digits;
         let mut smallest_digits = other.digits;
         if digits_lesser_than(&largest_digits, &smallest_digits) {
@@ -1067,7 +1028,7 @@ impl<Digit: GcdDigit, const SEPARATOR: char, const SHIFT: usize> Gcd
             }
             let smallest_digits_count = smallest_digits.len();
             if smallest_digits_count == 1 && smallest_digits[0].is_zero() {
-                return Self {
+                return Self::Output {
                     sign: Sign::one(),
                     digits: largest_digits,
                 };
@@ -1193,7 +1154,7 @@ impl<Digit: GcdDigit, const SEPARATOR: char, const SHIFT: usize> Gcd
             largest_digits = next_largest_digits;
             smallest_digits = next_smallest_digits;
         }
-        Self::from(
+        Self::Output::from(
             reduce_digits::<Digit, DoublePrecisionOf<Digit>, SHIFT>(&largest_digits).gcd(
                 reduce_digits::<Digit, DoublePrecisionOf<Digit>, SHIFT>(&smallest_digits),
             ),
@@ -1227,7 +1188,7 @@ impl<Digit, const SEPARATOR: char, const SHIFT: usize> Neg for BigInt<Digit, SEP
     type Output = Self;
 
     fn neg(self) -> Self::Output {
-        Self {
+        Self::Output {
             sign: -self.sign,
             digits: self.digits,
         }
@@ -1270,6 +1231,32 @@ impl<Digit: Clone + Eq + PartialOrd + ZeroableDigit, const SEPARATOR: char, cons
     }
 }
 
+impl<Source: Copy, Digit: PartialEq, const SEPARATOR: char, const SHIFT: usize> PartialEq<Source>
+    for BigInt<Digit, SEPARATOR, SHIFT>
+where
+    Self: From<Source>,
+{
+    fn eq(&self, other: &Source) -> bool {
+        self.eq(&Self::from(*other))
+    }
+}
+
+macro_rules! plain_eq_to_big_int_impl {
+    ($($t:ty)*) => ($(
+        impl<Digit: PartialEq, const SEPARATOR: char, const SHIFT: usize>
+            PartialEq<BigInt<Digit, SEPARATOR, SHIFT>> for $t
+        where
+            BigInt<Digit, SEPARATOR, SHIFT>: From<$t>,
+        {
+            fn eq(&self, other: &BigInt<Digit, SEPARATOR, SHIFT>) -> bool {
+                other.eq(&Self::from(*self))
+            }
+        }
+    )*)
+}
+
+plain_eq_to_big_int_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+
 impl<Digit: Clone + PartialOrd + ZeroableDigit, const SEPARATOR: char, const SHIFT: usize>
     PartialOrd for BigInt<Digit, SEPARATOR, SHIFT>
 {
@@ -1328,6 +1315,18 @@ impl<Digit: Clone + PartialOrd + ZeroableDigit, const SEPARATOR: char, const SHI
     }
 }
 
+impl<Digit: ParitiableDigit, const SEPARATOR: char, const SHIFT: usize> Parity
+    for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    fn is_even(&self) -> bool {
+        (self.digits[0] & Digit::one()).is_zero()
+    }
+
+    fn is_odd(&self) -> bool {
+        !(self.digits[0] & Digit::one()).is_zero()
+    }
+}
+
 impl<Digit: ExponentiativeDigit, const SEPARATOR: char, const SHIFT: usize> Pow<Self>
     for BigInt<Digit, SEPARATOR, SHIFT>
 {
@@ -1348,7 +1347,7 @@ impl<Digit: DivisibleDigit, const SEPARATOR: char, const SHIFT: usize> Rem
         let (sign, digits) =
             checked_rem::<Digit, SHIFT>(&self.digits, self.sign, &divisor.digits, divisor.sign)
                 .unwrap();
-        Self { sign, digits }
+        Self::Output { sign, digits }
     }
 }
 
@@ -1365,7 +1364,7 @@ impl<Digit: EuclidDivisibleDigit, const SEPARATOR: char, const SHIFT: usize> Rem
             divisor.sign,
         )
         .unwrap();
-        Self { sign, digits }
+        Self::Output { sign, digits }
     }
 }
 
@@ -1626,7 +1625,7 @@ impl<Digit: AdditiveDigit, const SEPARATOR: char, const SHIFT: usize> Sub
             &subtrahend.digits,
             subtrahend.sign,
         );
-        Self { sign, digits }
+        Self::Output { sign, digits }
     }
 }
 
@@ -1755,3 +1754,73 @@ impl<Digit: ZeroableDigit, const SEPARATOR: char, const SHIFT: usize> Zeroable
 const fn is_valid_shift<Digit: Oppose, const SHIFT: usize>() -> bool {
     SHIFT < 8 * size_of::<Digit>() - (utils::is_signed::<Digit>() as usize)
 }
+
+impl<Digit: ExponentiativeDigit, const SEPARATOR: char, const SHIFT: usize>
+    BigInt<Digit, SEPARATOR, SHIFT>
+{
+    fn unchecked_pow(self, exponent: &Self) -> Self {
+        debug_assert!(!exponent.is_negative());
+        let mut result = Self::one();
+        let mut exponent_digit = exponent.digits[exponent.digits.len() - 1];
+        if exponent.digits.len() == 1 && exponent_digit <= Digit::from(3) {
+            if exponent_digit >= Digit::from(2) {
+                result = self.clone() * self.clone();
+                if exponent_digit == Digit::from(3) {
+                    result *= self;
+                }
+            } else if exponent_digit.is_one() {
+                result *= self;
+            }
+        } else if exponent.digits.len() <= WINDOW_CUTOFF {
+            result = self.clone();
+            let mut exponent_digit_mask = Digit::from(2);
+            loop {
+                if exponent_digit_mask > exponent_digit {
+                    exponent_digit_mask >>= 1;
+                    break;
+                }
+                exponent_digit_mask <<= 1;
+            }
+            exponent_digit_mask >>= 1;
+            let mut exponent_digits_iterator = exponent.digits.iter().rev().skip(1);
+            loop {
+                while !exponent_digit_mask.is_zero() {
+                    result *= result.clone();
+                    if !(exponent_digit & exponent_digit_mask).is_zero() {
+                        result *= self.clone();
+                    }
+                    exponent_digit_mask >>= 1;
+                }
+                match exponent_digits_iterator.next() {
+                    Some(next_exponent_digit) => {
+                        exponent_digit = *next_exponent_digit;
+                        exponent_digit_mask = Digit::one() << (SHIFT - 1);
+                    }
+                    None => {
+                        break;
+                    }
+                }
+            }
+        } else {
+            let mut cache = vec![Self::zero(); WINDOW_BASE];
+            cache[0] = result.clone();
+            for index in 1..WINDOW_BASE {
+                cache[index] = cache[index - 1].clone() * self.clone();
+            }
+            let exponent_window_digits = binary_digits_to_lesser_binary_base::<Digit, WindowDigit>(
+                &exponent.digits,
+                SHIFT,
+                WINDOW_SHIFT,
+            );
+            for &digit in exponent_window_digits.iter().rev() {
+                for _ in 0..WINDOW_SHIFT {
+                    result *= result.clone();
+                }
+                if !digit.is_zero() {
+                    result *= cache[digit as usize].clone();
+                }
+            }
+        }
+        result
+    }
+}
