@@ -136,6 +136,8 @@ where
 
 pub trait OppositiveDigit = ZeroableDigit;
 
+pub trait ParitiableDigit = AssigningBitwiseConjunctiveMagma + Copy + UnitaryDigit;
+
 pub trait RightShiftableDigit =
     AdditiveDigit + Debug + DivisibleDigit + ShiftingRightMonoid + TryFrom<usize>
     where DoublePrecisionOf<Self>: AssigningShiftingLeftMonoid<Self>;
@@ -147,6 +149,7 @@ pub trait ZeroableDigit = Oppose + Zeroable;
 pub(crate) type Sign = i8;
 pub(crate) type WindowDigit = u8;
 
+#[derive(Eq, PartialEq)]
 pub enum CheckedDivApproximationError {
     TooLarge,
     ZeroDivision,
