@@ -9,10 +9,10 @@ use crate::digits::{
     MultiplicativeDigit, UnitaryDigit,
 };
 use crate::traits::{
-    Abs, AdditiveMonoid, CheckedDiv, CheckedDivAsF32, CheckedDivAsF64, CheckedDivEuclid,
-    CheckedPow, CheckedRemEuclid, CheckedShl, DivisivePartialMagma, Float, GcdMagma, Maybe,
-    ModularUnaryAlgebra, MultiplicativeMonoid, NegatableUnaryAlgebra, Oppositive, Pow,
-    SubtractiveMagma, Unitary, Zeroable,
+    Abs, AdditiveMonoid, Ceil, CheckedDiv, CheckedDivAsF32, CheckedDivAsF64, CheckedDivEuclid,
+    CheckedDivRemEuclid, CheckedPow, CheckedRemEuclid, CheckedShl, DivisivePartialMagma, Float,
+    Floor, GcdMagma, Maybe, ModularUnaryAlgebra, MultiplicativeMonoid, NegatableUnaryAlgebra,
+    Oppositive, Parity, Pow, Round, SubtractiveMagma, TieBreaking, Trunc, Unitary, Zeroable,
 };
 use crate::utils;
 
@@ -21,7 +21,7 @@ pub struct Fraction<Component: Clone + Eq> {
     numerator: Component,
     denominator: Component,
 }
-
+#[derive(Eq, PartialEq)]
 pub enum FromFloatConversionError {
     Infinity,
     NaN,
@@ -65,6 +65,15 @@ impl<Component: Clone + DivisivePartialMagma + Eq + GcdMagma + Oppositive> Fract
     }
 }
 
+impl<Component: Clone + Eq + Unitary> From<Component> for Fraction<Component> {
+    fn from(value: Component) -> Self {
+        Self {
+            numerator: value,
+            denominator: Component::one(),
+        }
+    }
+}
+
 impl<Component: Clone + Eq> Fraction<Component> {
     pub fn denominator(&self) -> &Component {
         &self.denominator
@@ -177,6 +186,26 @@ impl<
     }
 }
 
+impl<
+        Component: Clone
+            + CheckedDivEuclid<Output = Option<Component>>
+            + Eq
+            + MultiplicativeMonoid
+            + NegatableUnaryAlgebra
+            + Zeroable,
+    > Ceil for Fraction<Component>
+{
+    type Output = Component;
+
+    fn ceil(self) -> Self::Output {
+        -unsafe {
+            (-self.numerator)
+                .checked_div_euclid(self.denominator)
+                .unwrap_unchecked()
+        }
+    }
+}
+
 impl<
         Component: Clone + DivisivePartialMagma + Eq + GcdMagma + Oppositive + MultiplicativeMonoid,
     > CheckedDiv for Fraction<Component>
@@ -273,28 +302,20 @@ impl<
     type Output = Option<Component>;
 
     fn checked_div_euclid(self, divisor: Self) -> Self::Output {
-        if divisor.is_zero() {
-            None
-        } else {
-            (self.numerator * divisor.denominator)
-                .checked_div_euclid(divisor.numerator * self.denominator)
-        }
+        (self.numerator * divisor.denominator)
+            .checked_div_euclid(divisor.numerator * self.denominator)
     }
 }
 
 impl<
-        Component: Clone + CheckedDivEuclid<Output = Option<Component>> + Eq + MultiplicativeMonoid + Zeroable,
+        Component: Clone + CheckedDivEuclid<Output = Option<Component>> + Eq + MultiplicativeMonoid,
     > CheckedDivEuclid<Component> for Fraction<Component>
 {
     type Output = Option<Component>;
 
     fn checked_div_euclid(self, divisor: Component) -> Self::Output {
-        if divisor.is_zero() {
-            None
-        } else {
-            self.numerator
-                .checked_div_euclid(divisor * self.denominator)
-        }
+        self.numerator
+            .checked_div_euclid(divisor * self.denominator)
     }
 }
 
@@ -305,11 +326,7 @@ macro_rules! plain_checked_div_euclid_fraction_impl {
         type Output = Option<Self>;
 
         fn checked_div_euclid(self, divisor: Fraction<Self>) -> Self::Output {
-            if divisor.is_zero() {
-                None
-            } else {
-                (self * divisor.denominator).checked_div_euclid(divisor.numerator)
-            }
+            (self * divisor.denominator).checked_div_euclid(divisor.numerator)
         }
     }
     )*)
@@ -326,12 +343,113 @@ impl<
     type Output = Option<Self>;
 
     fn checked_div_euclid(self, divisor: Fraction<Self>) -> Self::Output {
-        if divisor.is_zero() {
-            None
-        } else {
-            (self * divisor.denominator).checked_div_euclid(divisor.numerator)
+        (self * divisor.denominator).checked_div_euclid(divisor.numerator)
+    }
+}
+
+impl<
+        Component: Clone
+            + CheckedDivRemEuclid<Output = Option<(Component, Component)>>
+            + DivisivePartialMagma
+            + Eq
+            + GcdMagma
+            + MultiplicativeMonoid
+            + Zeroable,
+    > CheckedDivRemEuclid for Fraction<Component>
+{
+    type Output = Option<(Component, Self)>;
+
+    fn checked_div_rem_euclid(self, divisor: Self) -> Self::Output {
+        let (quotient, remainder_numerator) = (self.numerator * divisor.denominator.clone())
+            .checked_div_rem_euclid(divisor.numerator * self.denominator.clone())?;
+        let (remainder_numerator, remainder_denominator) = normalize_components_moduli(
+            remainder_numerator,
+            self.denominator * divisor.denominator,
+        );
+        Some((
+            quotient,
+            Self {
+                numerator: remainder_numerator,
+                denominator: remainder_denominator,
+            },
+        ))
+    }
+}
+
+impl<
+        Component: Clone
+            + CheckedDivRemEuclid<Output = Option<(Component, Component)>>
+            + DivisivePartialMagma
+            + Eq
+            + GcdMagma
+            + MultiplicativeMonoid
+            + Zeroable,
+    > CheckedDivRemEuclid<Component> for Fraction<Component>
+{
+    type Output = Option<(Component, Self)>;
+
+    fn checked_div_rem_euclid(self, divisor: Component) -> Self::Output {
+        let (quotient, remainder_numerator) =
+            (self.numerator).checked_div_rem_euclid(divisor * self.denominator.clone())?;
+        let (remainder_numerator, remainder_denominator) =
+            normalize_components_moduli(remainder_numerator, self.denominator);
+        Some((
+            quotient,
+            Self {
+                numerator: remainder_numerator,
+                denominator: remainder_denominator,
+            },
+        ))
+    }
+}
+
+macro_rules! plain_checked_div_rem_euclid_fraction_impl {
+    ($($t:ty)*) => ($(
+    impl CheckedDivRemEuclid<Fraction<Self>> for $t
+    {
+        type Output = Option<(Self, Fraction<Self>)>;
+
+        fn checked_div_rem_euclid(self, divisor: Fraction<Self>) -> Self::Output {
+            let (quotient, remainder_numerator) = (self * divisor.denominator.clone())
+                .checked_div_rem_euclid(divisor.numerator)?;
+            let (remainder_numerator, remainder_denominator) = normalize_components_moduli(
+                remainder_numerator, divisor.denominator,
+            );
+            Some((
+                quotient,
+                Fraction::<Self> {
+                    numerator: remainder_numerator,
+                    denominator: remainder_denominator,
+                },
+            ))
         }
     }
+    )*)
+}
+
+plain_checked_div_rem_euclid_fraction_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+
+impl<
+        Digit: Eq + EuclidDivisibleDigit + GcdDigit + MultiplicativeDigit,
+        const SEPARATOR: char,
+        const SHIFT: usize,
+    > CheckedDivRemEuclid<Fraction<Self>> for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    type Output = Option<(Self, Fraction<Self>)>;
+
+    fn checked_div_rem_euclid(self, divisor: Fraction<Self>) -> Self::Output {
+        let (quotient, remainder_numerator) =
+            (self * divisor.denominator.clone()).checked_div_rem_euclid(divisor.numerator)?;
+        let (remainder_numerator, remainder_denominator) =
+            normalize_components_moduli(remainder_numerator, divisor.denominator);
+        Some((
+            quotient,
+            Fraction::<Self> {
+                numerator: remainder_numerator,
+                denominator: remainder_denominator,
+            },
+        ))
+    }
 }
 
 impl<
@@ -362,13 +480,34 @@ impl<
             }
         } else {
             Some(Self {
-                numerator: self.numerator.checked_pow(exponent.clone())?,
-                denominator: self.denominator.checked_pow(exponent)?,
+                numerator: unsafe {
+                    self.numerator
+                        .checked_pow(exponent.clone())
+                        .unwrap_unchecked()
+                },
+                denominator: unsafe { self.denominator.checked_pow(exponent).unwrap_unchecked() },
             })
         }
     }
 }
 
+macro_rules! plain_fraction_checked_pow_impl {
+    ($($t:ty)*) => ($(
+        impl CheckedPow<u32> for Fraction<$t> {
+            type Output = Option<Self>;
+
+            fn checked_pow(self, exponent: u32) -> Self::Output {
+                Some(Self {
+                    numerator: self.numerator.checked_pow(exponent)?,
+                    denominator: self.denominator.checked_pow(exponent)?,
+                })
+            }
+        }
+        )*)
+}
+
+plain_fraction_checked_pow_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+
 impl<
         Component: Clone
             + CheckedRemEuclid<Output = Option<Component>>
@@ -382,19 +521,15 @@ impl<
     type Output = Option<Self>;
 
     fn checked_rem_euclid(self, divisor: Self) -> Self::Output {
-        if divisor.is_zero() {
-            None
-        } else {
-            let (numerator, denominator) = normalize_components_moduli(
-                (self.numerator * divisor.denominator.clone())
-                    .checked_rem_euclid(divisor.numerator * self.denominator.clone())?,
-                self.denominator * divisor.denominator,
-            );
-            Some(Self {
-                numerator,
-                denominator,
-            })
-        }
+        let (numerator, denominator) = normalize_components_moduli(
+            (self.numerator * divisor.denominator.clone())
+                .checked_rem_euclid(divisor.numerator * self.denominator.clone())?,
+            self.denominator * divisor.denominator,
+        );
+        Some(Self {
+            numerator,
+            denominator,
+        })
     }
 }
 
@@ -411,19 +546,15 @@ impl<
     type Output = Option<Self>;
 
     fn checked_rem_euclid(self, divisor: Component) -> Self::Output {
-        if divisor.is_zero() {
-            None
-        } else {
-            let (numerator, denominator) = normalize_components_moduli(
-                self.numerator
-                    .checked_rem_euclid(divisor * self.denominator.clone())?,
-                self.denominator,
-            );
-            Some(Self {
-                numerator,
-                denominator,
-            })
-        }
+        let (numerator, denominator) = normalize_components_moduli(
+            self.numerator
+                .checked_rem_euclid(divisor * self.denominator.clone())?,
+            self.denominator,
+        );
+        Some(Self {
+            numerator,
+            denominator,
+        })
     }
 }
 
@@ -478,6 +609,27 @@ impl<
     }
 }
 
+impl<Component: Clone + Debug + Eq + Unitary> Debug for Fraction<Component> {
+    fn fmt(&self, formatter: &mut Formatter) -> std::fmt::Result {
+        if self.denominator.is_one() {
+            write!(
+                formatter,
+                "{}::from({:?})",
+                std::any::type_name::<Self>(),
+                self.numerator
+            )
+        } else {
+            write!(
+                formatter,
+                "{}::new({:?}, {:?}).unwrap()",
+                std::any::type_name::<Self>(),
+                self.numerator,
+                self.denominator
+            )
+        }
+    }
+}
+
 impl<Component: Clone + Display + Eq + Unitary> Display for Fraction<Component> {
     fn fmt(&self, formatter: &mut Formatter) -> std::fmt::Result {
         if self.denominator.is_one() {
@@ -535,6 +687,21 @@ impl<Digit: Eq + GcdDigit + MultiplicativeDigit, const SEPARATOR: char, const SH
     }
 }
 
+impl<
+        Component: Clone + CheckedDivEuclid<Output = Option<Component>> + Eq + MultiplicativeMonoid + Zeroable,
+    > Floor for Fraction<Component>
+{
+    type Output = Component;
+
+    fn floor(self) -> Self::Output {
+        unsafe {
+            self.numerator
+                .checked_div_euclid(self.denominator)
+                .unwrap_unchecked()
+        }
+    }
+}
+
 impl<
         Component: Clone + DivisivePartialMagma + Eq + GcdMagma + Oppositive + MultiplicativeMonoid,
     > Mul for Fraction<Component>
@@ -617,6 +784,16 @@ impl<Component: Clone + Eq + NegatableUnaryAlgebra> Neg for Fraction<Component>
     }
 }
 
+impl<Component: Clone + Eq + Oppositive + Unitary> Oppositive for Fraction<Component> {
+    fn is_negative(&self) -> bool {
+        self.numerator.is_negative()
+    }
+
+    fn is_positive(&self) -> bool {
+        self.numerator.is_positive()
+    }
+}
+
 impl<Component: Clone + Eq + MultiplicativeMonoid + PartialOrd> Ord for Fraction<Component> {
     fn cmp(&self, other: &Self) -> Ordering {
         if self.lt(other) {
@@ -815,6 +992,46 @@ impl<
     }
 }
 
+impl<
+        Component: AdditiveMonoid
+            + Clone
+            + CheckedDivRemEuclid<Output = Option<(Component, Component)>>
+            + Eq
+            + From<u8>
+            + MultiplicativeMonoid
+            + Oppositive
+            + Ord
+            + Parity
+            + SubtractiveMagma,
+    > Round for Fraction<Component>
+{
+    type Output = Component;
+
+    fn round(self, tie_breaking: TieBreaking) -> Self::Output {
+        let (quotient, remainder) = unsafe {
+            self.numerator
+                .checked_div_rem_euclid(self.denominator.clone())
+                .unwrap_unchecked()
+        };
+        match (remainder * Component::from(2u8)).cmp(&self.denominator) {
+            Ordering::Equal => {
+                if match tie_breaking {
+                    TieBreaking::AwayFromZero => !quotient.is_negative(),
+                    TieBreaking::ToEven => quotient.is_odd(),
+                    TieBreaking::ToOdd => quotient.is_even(),
+                    TieBreaking::TowardZero => quotient.is_negative(),
+                } {
+                    quotient + Component::one()
+                } else {
+                    quotient
+                }
+            }
+            Ordering::Greater => quotient + Component::one(),
+            Ordering::Less => quotient,
+        }
+    }
+}
+
 impl<
         Component: Clone
             + DivisivePartialMagma
@@ -871,7 +1088,7 @@ macro_rules! plain_sub_fraction_impl {
 
         fn sub(self, subtrahend: Fraction<Self>) -> Self::Output {
             let (numerator, denominator) = normalize_components_moduli(
-                self * subtrahend.denominator.clone() - subtrahend.numerator,
+                self * subtrahend.denominator - subtrahend.numerator,
                 subtrahend.denominator,
             );
             Self::Output {
@@ -921,6 +1138,25 @@ impl<
     }
 }
 
+impl<
+        Component: Clone
+            + CheckedDivEuclid<Output = Option<Component>>
+            + Eq
+            + MultiplicativeMonoid
+            + Oppositive,
+    > Trunc for Fraction<Component>
+{
+    type Output = Component;
+
+    fn trunc(self) -> Self::Output {
+        if self.is_negative() {
+            self.ceil()
+        } else {
+            self.floor()
+        }
+    }
+}
+
 impl<Component: Clone + Eq + CheckedDivAsF32> TryInto<f32> for Fraction<Component> {
     type Error = <<Component as CheckedDivAsF32>::Output as Maybe>::Error;
 
