@@ -97,10 +97,31 @@ macro_rules! plain_abs_impl {
 
 plain_abs_impl!(f32 f64 i8 i16 i32 i64 i128 isize);
 
-#[derive(Clone)]
+pub trait Ceil {
+    type Output;
+
+    fn ceil(self) -> Self::Output;
+}
+
+macro_rules! plain_ceil_impl {
+    ($($t:ty)*) => ($(
+        impl Ceil for $t {
+            type Output = $t;
+
+            #[inline(always)]
+            fn ceil(self) -> Self::Output {
+                <$t>::ceil(self)
+            }
+        }
+    )*)
+}
+
+plain_ceil_impl!(f32 f64);
+
+#[derive(Clone, Eq, PartialEq)]
 pub enum Endianness {
-    BIG,
-    LITTLE,
+    Big,
+    Little,
 }
 
 pub trait FromBytes {
@@ -113,8 +134,8 @@ macro_rules! plain_from_bytes_impl {
             #[inline(always)]
             fn from_bytes(bytes: &[u8], endianness: Endianness) -> Self {
                 match endianness {
-                   Endianness::BIG => Self::from_be_bytes(bytes.try_into().unwrap()),
-                   Endianness::LITTLE => Self::from_le_bytes(bytes.try_into().unwrap()),
+                   Endianness::Big => Self::from_be_bytes(bytes.try_into().unwrap()),
+                   Endianness::Little => Self::from_le_bytes(bytes.try_into().unwrap()),
                 }
             }
         }
@@ -137,8 +158,8 @@ macro_rules! plain_to_bytes_impl {
             #[inline(always)]
             fn to_bytes(self, endianness: Endianness) -> Self::Output {
                 match endianness {
-                   Endianness::BIG => self.to_be_bytes(),
-                   Endianness::LITTLE => self.to_le_bytes(),
+                   Endianness::Big => self.to_be_bytes(),
+                   Endianness::Little => self.to_le_bytes(),
                 }
             }
         }
@@ -816,7 +837,7 @@ impl FrExp for f64 {
     type Output = (Self, i32);
 
     fn frexp(self) -> Self::Output {
-        let mut bits = self.to_bits();
+        let bits = self.to_bits();
         let exponent_bits = ((bits >> 52) & 0x7ff) as i32;
         if exponent_bits.is_zero() {
             if !self.is_zero() {
@@ -829,10 +850,10 @@ impl FrExp for f64 {
         } else if exponent_bits == 0x7ff {
             (self, 0)
         } else {
-            let e = exponent_bits - 0x3fe;
-            bits &= 0x800fffffffffffff;
-            bits |= 0x3fe0000000000000;
-            (f64::from_bits(bits), e)
+            (
+                f64::from_bits(bits & 0x800fffffffffffff | 0x3fe0000000000000),
+                exponent_bits - 0x3fe,
+            )
         }
     }
 }
@@ -841,7 +862,7 @@ impl FrExp for f32 {
     type Output = (Self, i32);
 
     fn frexp(self) -> Self::Output {
-        let mut bits = self.to_bits();
+        let bits = self.to_bits();
         let exponent_bits = ((bits >> 23) & 0xff) as i32;
         if exponent_bits.is_zero() {
             if !self.is_zero() {
@@ -854,10 +875,10 @@ impl FrExp for f32 {
         } else if exponent_bits == 0xff {
             (self, 0)
         } else {
-            let exponent = exponent_bits - 0x7e;
-            bits &= 0x807fffff;
-            bits |= 0x3f000000;
-            (f32::from_bits(bits), exponent)
+            (
+                f32::from_bits(bits & 0x807fffff | 0x3f000000),
+                exponent_bits - 0x7e,
+            )
         }
     }
 }
@@ -1159,6 +1180,30 @@ macro_rules! plain_oppositive_impl {
 
 plain_oppositive_impl!(i8 i16 i32 i64 i128 isize);
 
+pub trait Parity {
+    fn is_even(&self) -> bool;
+
+    fn is_odd(&self) -> bool;
+}
+
+macro_rules! plain_parity_impl {
+    ($($t:ty)*) => ($(
+        impl Parity for $t {
+            #[inline(always)]
+            fn is_even(&self) -> bool {
+                self & 1 == 0
+            }
+
+            #[inline(always)]
+            fn is_odd(&self) -> bool {
+                self & 1 == 1
+            }
+        }
+    )*)
+}
+
+plain_parity_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+
 pub trait Pow<Exponent> {
     type Output;
 
@@ -1201,6 +1246,75 @@ macro_rules! plain_rem_euclid_impl {
 
 plain_rem_euclid_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
 
+#[derive(Clone, Eq, PartialEq)]
+pub enum TieBreaking {
+    AwayFromZero,
+    ToEven,
+    ToOdd,
+    TowardZero,
+}
+
+pub trait Round {
+    type Output;
+
+    fn round(self, tie_breaking: TieBreaking) -> Self::Output;
+}
+
+macro_rules! plain_round_impl {
+    ($($t:ty)*) => ($(
+        impl Round for $t {
+            type Output = $t;
+
+            #[inline(always)]
+            fn round(self, tie_breaking: TieBreaking) -> Self::Output {
+                match tie_breaking {
+                    TieBreaking::AwayFromZero => <$t>::round(self),
+                    TieBreaking::ToEven => {
+                        if self.ceil() - self == (0.5 as $t) {
+                            2.0 * <$t>::round(self / 2.0)
+                        } else {
+                            <$t>::round(self)
+                        }
+                    }
+                    TieBreaking::ToOdd => {
+                        if self.ceil() - self == (0.5 as $t) {
+                            2.0 * (self / 2.0).floor() + 1.0
+                        } else {
+                            <$t>::round(self)
+                        }
+                    }
+                    TieBreaking::TowardZero => {
+                        self.trunc() + ((self.fract().abs() > (0.5 as $t)) as i32 as $t) * self.signum()
+                    }
+                }
+            }
+        }
+    )*)
+}
+
+plain_round_impl!(f32 f64);
+
+pub trait Trunc {
+    type Output;
+
+    fn trunc(self) -> Self::Output;
+}
+
+macro_rules! plain_trunc_impl {
+    ($($t:ty)*) => ($(
+        impl Trunc for $t {
+            type Output = $t;
+
+            #[inline(always)]
+            fn trunc(self) -> Self::Output {
+                <$t>::trunc(self)
+            }
+        }
+    )*)
+}
+
+plain_trunc_impl!(f32 f64);
+
 pub trait Unitary {
     fn one() -> Self;
 
